import sys
import os
import math
import re
import random
import time
import math

def ensureDirectory(d):
    if not os.path.exists(d):
        os.makedirs(d)

def usage():
    usageStr = "Usage: python WeightMC.py [options] <inputFile>\n"
    usageStr += "Options are entered as '-object=value' where object can be: \n"
    usageStr += "epsilon (required): desired tolerance\n"
    usageStr += "delta (required): desired error probability (1 - confidence)\n"
    usageStr += "r (required): an upper bound on the tilt\n"
    usageStr += "timeout: overall timeout in seconds (default: 72000)\n"
    usageStr += "satTimeout: timeout for a single BoundedWeightSAT call in seconds (default: 3000)\n"
    usageStr += "logging: 0 to turn off logging and 1 to turn it on.\n"
    usageStr += "log: folder to store logs in (default: logging)\n"
    usageStr += "runIndex: number appended to names of output & log files (default: current time) \n"
    print usageStr
    exit(1)

def getInputs():
    paramMap={}
    action=0
    error = ''
    acceptedParams=['timeout','satTimeout','runIndex','r','log','logging','epsilon','delta']
    for i in range(1,len(sys.argv)):
        if (not(sys.argv[i][0] == '-')):
            paramMap['inputFile'] = sys.argv[i]
            if (paramMap.has_key('epsilon') and paramMap.has_key('delta') and paramMap.has_key('r')):
                action = 3
                return action,error,paramMap
            else:
                action = 2
                error = "Required parameter 'epsilon', 'delta', or 'r' not supplied."
                return action,error,paramMap
        else:
            if (sys.argv[i][1] == 'h'):
                action = 0
                return action,error,paramMap
            fieldValues = sys.argv[i][1:].strip().split('=')
            if (not(fieldValues[0] in acceptedParams)):
                action = 1
                error = "Could not understand the option "+str(fieldValues[0])+"\n"
                return action,error,paramMap
            else:
                paramMap[fieldValues[0]] = fieldValues[1]
    action = 2
    error = "You must specify an input file. Use the option -h for help.\n"
    return action,error,paramMap

def FindFromTable(delta):
    if (not(os.path.isfile('ProbMapFile.txt'))):
        return 0
    f = open('ProbMapFile.txt','r')
    lines = f.readlines()
    f.close()
    for line in lines:
        fields = line.strip().split(':')
        if (float(fields[1]) > delta):
            return int(fields[0])
    return 0

def main():
    runIndex = str(int(time.time()))
    timeout=72000
    satTimeout = 3000
    initialFileName = ''
    numVariables = 0
    numClauses = 0
    pivotAC=0
    shouldLog = False
    outputFileName = ''
    epsilon = 0
    pivotUniGen = 0
    logFolder = None
    kappa = 0
    samples = 1
    action,error,paramMap = getInputs()
    if (action == 0):
        usage()
        exit(1)
    if (action == 1 or action == 2):
        print error
        exit(1)
    if (paramMap.has_key('runIndex')):
        runIndex = int(paramMap['runIndex'])
    if (paramMap.has_key('timeout')):
        timeout = float(paramMap['timeout'])
    if (paramMap.has_key('satTimeout')):
        satTimeout = float(paramMap['satTimeout'])
    if (paramMap.has_key('epsilon')):
        epsilon = float(paramMap['epsilon'])
    if (paramMap.has_key('delta')):
        delta = float(paramMap['delta'])
    if (paramMap.has_key('r')):
        r = float(paramMap['r'])
    if (paramMap.has_key('log')):
        logFolder = paramMap['log']
    if (paramMap.has_key('logging')):
        if (paramMap['logging'] == '0'):
            shouldLog = False
        if (paramMap['logging'] == '1'):
            shouldLog = True
    initialFileName = paramMap['inputFile']
    initialFileNameSuffix =  initialFileName.split('/')[-1][:-4]
    if (shouldLog):
        if not logFolder:
            logFolder = "logging"
        ensureDirectory(logFolder)
        logFileName = logFolder+"/"+str(initialFileNameSuffix)+'_'+str(runIndex)+'.txt'
        g = open(logFileName,'w')
        g.close()

    pivotAC = 2*math.ceil(4.4817*(1+1/epsilon)*(1+1/epsilon))
    numIterations = FindFromTable(1-delta)
    print delta, numIterations
    if (numIterations == 0):
        numIterations = int(math.ceil(35*math.log((3*1.0/delta),2)))

    cmd = "./weightmc --maxTotalTime="+str(timeout)+" --startIteration=0 --maxLoopTime="+str(satTimeout)+" --tApproxMC="+str(numIterations)+" --pivotAC="+str(pivotAC)+" --gaussuntil=400 --verbosity=0 --ratio="+str(r)
    if (shouldLog):
        cmd += " --logFile="+logFileName
    cmd += " "+initialFileName
    print cmd
    os.system(cmd)

main()    
